/*
*  Project     Nintendo Extension Controller Library
*  @author     David Madison
*  @link       github.com/dmadison/NintendoExtensionCtrl
*  @license    LGPLv3 - Copyright (c) 2018 David Madison
*
*  This file is part of the Nintendo Extension Controller Library.
*
*  This program is free software: you can redistribute it and/or modify
*  it under the terms of the GNU Lesser General Public License as published by
*  the Free Software Foundation, either version 3 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU Lesser General Public License for more details.
*
*  You should have received a copy of the GNU Lesser General Public License
*  along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "DJTurntable.h"

namespace NintendoExtensionCtrl {

constexpr ControlDataMap::ByteMap DJTurntableController_Data::Maps::JoyX;
constexpr ControlDataMap::ByteMap DJTurntableController_Data::Maps::JoyY;

constexpr ControlDataMap::BitMap  DJTurntableController_Data::Maps::ButtonPlus;
constexpr ControlDataMap::BitMap  DJTurntableController_Data::Maps::ButtonMinus;

constexpr ControlDataMap::ByteMap DJTurntableController_Data::Maps::Left_Turntable;
constexpr ControlDataMap::ByteMap DJTurntableController_Data::Maps::Left_TurntableSign;
constexpr ControlDataMap::BitMap  DJTurntableController_Data::Maps::Left_ButtonGreen;
constexpr ControlDataMap::BitMap  DJTurntableController_Data::Maps::Left_ButtonRed;
constexpr ControlDataMap::BitMap  DJTurntableController_Data::Maps::Left_ButtonBlue;

constexpr ControlDataMap::ByteMap DJTurntableController_Data::Maps::Right_Turntable[3];
constexpr ControlDataMap::ByteMap DJTurntableController_Data::Maps::Right_TurntableSign;
constexpr ControlDataMap::BitMap  DJTurntableController_Data::Maps::Right_ButtonGreen;
constexpr ControlDataMap::BitMap  DJTurntableController_Data::Maps::Right_ButtonRed;
constexpr ControlDataMap::BitMap  DJTurntableController_Data::Maps::Right_ButtonBlue;

constexpr ControlDataMap::ByteMap DJTurntableController_Data::Maps::EffectDial[2];
constexpr ControlDataMap::ByteMap DJTurntableController_Data::Maps::CrossfadeSlider;

constexpr ControlDataMap::BitMap  DJTurntableController_Data::Maps::ButtonEuphoria;

// Combined Turntable
int8_t DJTurntableController_Data::turntable() const {
	return left.turntable() + right.turntable();
}

boolean DJTurntableController_Data::buttonGreen() const {
	return left.buttonGreen() | right.buttonGreen();
}

boolean DJTurntableController_Data::buttonRed() const {
	return left.buttonRed() | right.buttonRed();
}

boolean DJTurntableController_Data::buttonBlue() const {
	return left.buttonBlue() | right.buttonBlue();
}

// Main Board
uint8_t DJTurntableController_Data::effectDial() const {
	return getControlData(Maps::EffectDial);
}

int8_t DJTurntableController_Data::crossfadeSlider() const {
	return getControlData(Maps::CrossfadeSlider) - 8;  // Shifted to signed int
}

boolean DJTurntableController_Data::buttonEuphoria() const {
	return getControlBit(Maps::ButtonEuphoria);
}

uint8_t DJTurntableController_Data::joyX() const {
	return getControlData(Maps::JoyX);
}

uint8_t DJTurntableController_Data::joyY() const {
	return getControlData(Maps::JoyY);
}

boolean DJTurntableController_Data::buttonPlus() const {
	return getControlBit(Maps::ButtonPlus);
}

boolean DJTurntableController_Data::buttonMinus() const {
	return getControlBit(Maps::ButtonMinus);
}

DJTurntableController_Data::TurntableConfig DJTurntableController_Data::getTurntableConfig() {
	if (tableConfig == TurntableConfig::Both) {
		return tableConfig;  // Both are attached, no reason to check data
	}

	boolean leftState = left.connected();
	boolean rightState = right.connected();

	if (leftState && rightState) {
		return tableConfig = TurntableConfig::Both;
	}
	else if (leftState) {
		return tableConfig = TurntableConfig::Left;
	}
	else if (rightState) {
		return tableConfig = TurntableConfig::Right;
	}
	else {
		return tableConfig = TurntableConfig::BaseOnly;
	}
}

uint8_t DJTurntableController_Data::getNumTurntables() {
	getTurntableConfig();  // Update config from data

	switch (tableConfig) {
		case TurntableConfig::BaseOnly:
			return 0;
			break;
		case TurntableConfig::Left:
		case TurntableConfig::Right:
			return 1;
			break;
		case TurntableConfig::Both:
			return 2;
			break;
	}
	return 0;  // Just in-case
}

void DJTurntableController_Data::printDebug(Print& output) {
	const char fillCharacter = '_';

	char buffer[45];

	output.print("DJ:");

	if (getNumTurntables() == 0) {
		output.print(" No turntable found! |");
	}
	else if (left.connected()) {
		printTurntable(output, left);
		output.print(" |");
	}

	char plusPrint = buttonPlus() ? '+' : fillCharacter;
	char minusPrint = buttonMinus() ? '-' : fillCharacter;

	char euphoriaPrint = buttonEuphoria() ? 'E' : fillCharacter;

	sprintf(buffer,
		" Joy:(%2u, %2u) | %c | %c%c | FX: %2u | Fade: %2d",
		joyX(), joyY(),
		euphoriaPrint,
		minusPrint, plusPrint,
		effectDial(), crossfadeSlider());
	output.print(buffer);

	if (right.connected()) {
		output.print(" |");
		printTurntable(output, right);
	}
	output.println();
}

void DJTurntableController_Data::printTurntable(Print& output, TurntableExpansion &table) const {
	const char fillCharacter = '_';

	char idPrint = 'X';
	if (table.side == TurntableConfig::Left) {
		idPrint = 'L';
	}
	else if (table.side == TurntableConfig::Right) {
		idPrint = 'R';
	}

	char greenPrint = table.buttonGreen() ? 'G' : fillCharacter;
	char redPrint = table.buttonRed() ? 'R' : fillCharacter;
	char bluePrint = table.buttonBlue() ? 'B' : fillCharacter;

	char buffer[15];
	sprintf(buffer,
		" T%c:%3d %c%c%c",
		idPrint,
		table.turntable(),
		greenPrint, redPrint, bluePrint);
	output.print(buffer);
}

// Turntable Expansion Base
boolean DJTurntableController_Data::TurntableExpansion::connected() const {
	if (base.tableConfig == TurntableConfig::Both || base.tableConfig == side) {
		return true;  // Already checked
	}
	return turntable() != 0 || buttonGreen() || buttonRed() || buttonBlue();
}

// Left Turntable
int8_t DJTurntableController_Data::TurntableLeft::turntable() const {
	uint8_t turnData = base.getControlData(Maps::Left_Turntable);
	boolean turnSign = base.getControlData(Maps::Left_TurntableSign);
	return getTurntableSpeed(turnData, turnSign);
}

boolean DJTurntableController_Data::TurntableLeft::buttonGreen() const {
	return base.getControlBit(Maps::Left_ButtonGreen);
}

boolean DJTurntableController_Data::TurntableLeft::buttonRed() const {
	return base.getControlBit(Maps::Left_ButtonRed);
}

boolean DJTurntableController_Data::TurntableLeft::buttonBlue() const {
	return base.getControlBit(Maps::Left_ButtonBlue);
}

// Right Turntable
int8_t DJTurntableController_Data::TurntableRight::turntable() const {
	uint8_t turnData = base.getControlData(Maps::Right_Turntable);
	boolean turnSign = base.getControlData(Maps::Right_TurntableSign);
	return getTurntableSpeed(turnData, turnSign);
}

boolean DJTurntableController_Data::TurntableRight::buttonGreen() const {
	return base.getControlBit(Maps::Right_ButtonGreen);
}

boolean DJTurntableController_Data::TurntableRight::buttonRed() const {
	return base.getControlBit(Maps::Right_ButtonRed);
}

boolean DJTurntableController_Data::TurntableRight::buttonBlue() const {
	return base.getControlBit(Maps::Right_ButtonBlue);
}

// Effect Rollover
int8_t DJTurntableController_Data::EffectRollover::getChange() {
	return RolloverChange::getChange(dj.effectDial());
}

}  // End "NintendoExtensionCtrl" namespace

